module vga_timing(
    input clk,
    input reset,
    output reg [11:0] x,
    output reg [11:0] y,
    output reg blank,
    output reg hsync,
    output reg vsync);

  // parameters for VGA
  parameter VISIBLE_AREA_H = 640;
  parameter FRONT_PORCH_H = 16;
  parameter SYNC_PULSE_H = 96;
  parameter BACK_PORCH_H = 48;
  parameter SYNC_POLARITY_H = 1'b0; // hsync polarity, 0 is negative
  localparam WHOLE_H = VISIBLE_AREA_H + FRONT_PORCH_H + SYNC_PULSE_H;

  parameter VISIBLE_AREA_V = 480;
  parameter FRONT_PORCH_V = 10;
  parameter SYNC_PULSE_V = 2;
  parameter BACK_PORCH_V = 33;
  parameter SYNC_POLARITY_V = 1'b0; // vsync pulse polarity, 0 is negative
  localparam WHOLE_V = VISIBLE_AREA_V + FRONT_PORCH_V + SYNC_PULSE_V;

  parameter BLANK_POLARITY = 1'b1; // blanking signal polarity, 0 is negative

  parameter LATENCY = 1'b1;

  reg [11:0] xp = 0;
  reg [11:0] yp = 0;

  // generate the current position
  always @(posedge clk) begin
    if (reset) begin
      xp = 0;
      yp = 0;
    end else begin
      if (xp == WHOLE_H-1) begin  // end of the current line
        if (yp == WHOLE_V-1) yp = 0;  // end of the frame, return to the beginning
        else yp = yp + 12'd1;  // advance to the next line
        xp = 0;  // return to the beginning of the line
      end else xp = xp + 12'd1;
    end
  end
  
  always @(posedge clk) begin
    // blanking indicates when we are inside the visible area
    if (xp < VISIBLE_AREA_H && yp < VISIBLE_AREA_V)
      blank <= ~BLANK_POLARITY;
    else
      blank = BLANK_POLARITY;
    
    // generate horizontal synchronization
    if (xp >= VISIBLE_AREA_H + FRONT_PORCH_H && xp < VISIBLE_AREA_H + FRONT_PORCH_H + SYNC_PULSE_H)
      hsync <= SYNC_POLARITY_H;
    else
      hsync <= ~SYNC_POLARITY_H; 

    // generate vertical synchronization
    if (yp >= VISIBLE_AREA_V + FRONT_PORCH_V && yp < VISIBLE_AREA_V + FRONT_PORCH_V + SYNC_PULSE_V)
      vsync <= SYNC_POLARITY_V;
    else
      vsync <= ~SYNC_POLARITY_V; 

    // generate the output positions
    x <= xp - LATENCY;
    y <= yp;
  end
endmodule
